<?php

if (!defined('_ECRIRE_INC_VERSION')) {
	return;
}

if (!defined('_SIMPLOG_LIGNE')) {
	/**
	 * Regexp permettant d'isoler les divers éléments d'un log spip.
	 */
	define('_SIMPLOG_LIGNE', '#^(.*:\d\d)\s(.*)\s\(pid\s(.*)\)\s(?:(.+):L(\d+):(\w+\(\)):)?:([bipru]*):([^:]*):\s(.*)$#i');
}

/**
 * Lit un fichier de log et lance son phrasage.
 *
 * @uses simplog_phraser_ligne()
 *
 * @param string $fichier Chemin complet du fichier de log
 *
 * @return array Liste des lignes du log entièrement phrasées.
 */
function simplog_phraser_log(string $fichier) : array {
	$tableau = [];

	$contenu = spip_file_get_contents($fichier);
	if ($contenu) {
		$tableau = simplog_phraser_ligne(preg_split('/\r?\n/', $contenu));
	}

	return $tableau;
}

/**
 * Phrase chaque ligne du contenu d'un log et renvoie un tableau associatif des éléments.
 *
 * @uses simplelog_contruire_texte()
 * @uses simplelog_finir_markup_texte()
 *
 * @param array $contenu Contenu du fichier de log sous la forme d'une liste de lignes non phrasées
 *
 * @return array Liste des lignes du log entièrement phrasées.
 */
function simplog_phraser_ligne(array $contenu) : array {
	$tableau = [];

	foreach ($contenu as $_ligne) {
		if (
			($l = trim($_ligne))
			and ($l != '[-- rotate --]')
		) {
			preg_match(_SIMPLOG_LIGNE, $_ligne, $matches);
			if (
				!isset($matches[1])
				or !$matches[1]
			) {
				// Ce n'est pas une nouvelle ligne mais la suite du texte de la ligne en cours
				$tableau[count($tableau) - 1]['texte'] .= "\n" . $_ligne;
			} else {
				$ligne['date'] = date('Y-m-d H:i:s', strtotime($matches[1]));
				$ligne['ip'] = trim($matches[2]);
				$ligne['pid'] = trim($matches[3]);
				$ligne['hit'] = _T('simplog:info_hit_' . strtolower(trim($matches[7])));
				$ligne['gravite'] = strtolower(trim($matches[8]));
				$ligne['texte'] = simplelog_contruire_texte(trim($matches[4]), trim($matches[5]), trim($matches[6]), $matches[9]);
				$ligne['index'] = count($tableau);
				$tableau[] = $ligne;
			}
		}
	}

	// Clore la balise <code> du message d'explication
	$tableau = array_map('simplelog_finir_markup_texte', $tableau);

	return $tableau;
}

/**
 * Construit la structure du texte pour le présenter de la façon la plus lisible possible.
 *
 * @param string $fichier  Nom du fichier d'où provient le log
 * @param string $ligne    Ligne d'où provient le log
 * @param string $fonction La fonction dans laquelle le log est issu
 * @param string $message  Texte complémentaire d'explication
 *
 * @return string Texte complet du log
 */
function simplelog_contruire_texte(string $fichier, string $ligne, string $fonction, string $message) : string {
	$label_fichier = _T('simplog:label_fichier');
	$label_ligne = _T('simplog:label_ligne');
	$label_fonction = _T('simplog:label_fonction');
	$texte = ''
		 . ($fichier ? "-*<em>{$label_fichier}</em> : {$fichier}" : '')
		 . ($ligne ? "\n-*<em>{$label_ligne}</em> : {$ligne}" : '')
		 . ($fonction ? "\n-*<em>{$label_fonction}</em> : {$fonction}" : '')
		 . "\n"
		 . ($message ? "<br /><code>\n\r{$message}" : '<code>');

	return $texte;
}

/**
 * Complète les textes des messages d'explication en fermant la balise `<code>`.
 *
 * @param array $ligne Tableau des éléments de la ligne en cours de fermeture.
 *
 * @return array
 */
function simplelog_finir_markup_texte(array $ligne) : array {
	$ligne['texte'] = unicode2charset($ligne['texte'] . "\n\r</code>");

	return $ligne;
}
